'// User defined type example
#include once "../fbList.bi"

'// Create our UDT with custom Compare, and CSV functions.
type myType
	foo as integer
	declare function myMethod() as integer
	
	declare function Compare(byref b as myType) as integer
	declare function CSV() as string
end type
function myType.myMethod() as integer
	return this.foo * 3
end function

'// The custom compare is used by SORTing, and FIND.
'// If it does not exist then SORTing, and FIND, fall back to
'//  comparing the memory address of the element, which is rarely accurate.
'// If you do not need SORTing, or FIND, then it is safe to not
'//  create the Compare method.
function myType.Compare(byref b as myType) as integer
	if this.foo < b.foo then
		return -1
	elseif this.foo > b.foo then
		return 1
	end if
	return 0
end function
'// The type.CSV function is used by list.CSV
'// If it does not exist then list.CSV will simply output the memory address.
'// If you do not need list.CSV, or if you are fine with getting a list of
'//  memory addresses then it is safe to not create the CSV method.
function myType.CSV() as string
	return str(this.foo)
end function

'// Create a new list type named myUDTList that can hold myType
TypeList myUDTList Holds myType
'// Use our TYPELIST to create a list
dim as myUDTList myList


dim as myType aValue
aValue.foo = 1

'// There are some considerations and methods that must be employed when
'//  inserting copies of more complex user defined types.
'// See example-08 for more information.
myList += aValue '// Insert a copy

'// Since this only inserts a ptr reference,
'//  we do not need to worry about the complexity of the user defined type.
myList += @aValue '// Insert a reference

'// At this point both items in the list will be 1
print "After +=, myList is "& myList.Length &" items long:"
print "	"& myList.CSV()

'// Since we inserted a reference this will have conseqeucnes
aValue = type(10) '// Set 'aValue' to 1

'// At this point, since we have changed 'aValue', and the second item in the list
'//  at index 1 is a reference, the values will now be 1, 10
print "After 'aValue' was changed myList is "& myList.Length &" items long:"
print "	"& myList.CSV()
print " (Note how the second value has changed because it is a reference)"


print "We can directly access an element via [] or GetPtr"
print "	[0].foo:        "& myList[0].foo
print "	GetPtr(0)->foo: "& myList.GetPtr(0)->foo
print "	[1].foo:        "& myList[1].foo
print "	GetPtr(1)->foo: "& myList.GetPtr(1)->foo

print "Accessing type methods is just as simple"
print "	[0].myMethod:        "& myList[0].myMethod
print "	GetPtr(0)->myMethod: "& myList.GetPtr(0)->myMethod
print "	[1].myMethod:        "& myList[1].myMethod
print "	GetPtr(1)->myMethod: "& myList.GetPtr(1)->myMethod

print
print "Press any key to close the example!"
sleep
