'// Advanced user defined type example
#include once "../fbList.bi"

'// User defined types that have manually allocated memory can not be copied
'//  by simply doing a = b, additional steps are required to allocate new
'//  memory and then copying the contents over.
print "Using a custom .Copy method for manually allocated memory!"
print

'// Create our UDT with custom Copy, and CSV, functions.
type myTypeMemory
	foo as integer ptr
	
	declare function CSV() as string
	
'// To ensure our UDT is properly and fully copied we declare a custom
'//  Copy method that will be automatically used when inserting a copy
'//  into a list.
	declare function Copy() as myTypeMemory ptr
	
	declare destructor
end type
function myTypeMemory.CSV() as string
	return "0x"& hex(this.foo)
end function
destructor myTypeMemory '// Prevent memory leaks by deallocating our manual allocs
	if this.foo then
		deallocate this.foo
	end if
	this.foo = 0
end destructor

function myTypeMemory.Copy() as myTypeMemory ptr
	dim as myTypeMemory ptr newcopy = new myTypeMemory '// Create a new object
	*newcopy = this		'// Copy over any basic non-ptr types such as integer, double, string, etc
	
	if this.foo then	'// We know this is a ptr, so we must manually copy it.
						'//  let's allocate a new memory block and copy it over
		newcopy->foo = allocate(sizeof(integer))	'// Create our new memory block
		*newcopy->foo = *this.foo					'// And copy the existing value
	end if
	return newcopy
end function

'// Create a new list type named myMemoryList that can hold myTypeMemory
TypeList myMemoryList Holds myTypeMemory
'// Use our TYPELIST to create a list
dim as myMemoryList List1
dim as myTypeMemory myValue
myValue.foo	=	allocate(sizeof(integer))
*myValue.foo	=	12345

List1 += myValue	'// Insert a copy
List1 += @myValue	'// Insert a reference

print "	myValue.foo points to 0x"& hex(myValue.foo)

print "	List1 ["& List1.Length &" items]:"
print "		"& List1.CSV()

'// We deallocate myValue.foo here
deallocate myValue.foo
myValue.foo	=	0

print "	After deallocate, myValue.foo points to 0x"& hex(myValue.foo)

'// The .foo that was copied into the list, using our custom .Copy method, is still valid
print "	But the copy in List1 is still valid ["& List1.Length &" items]:"
print "		"& List1.CSV()
print "			(Note that the second entry is now 0x0 since it's a reference and not a copy)"

print "	The inserted copy is still valid and can be dereferenced: "& *List1[0].foo

'// A custom .Copy is only needed if you manually allocate memory in your
'//  used defined type. If your user defined type contains no pointers/references
'//  then it is safe to not create the Copy method

'// You can toggle the use of your custom .Copy method via
List1.DisableCopy()
List1.EnableCopy()
'// Default is Enabled



print
print "Press any key to close the example!"
sleep
